#!/usr/bin/python3

import glob
import os
import sys,getopt
import subprocess
import time

cpus = []
mid_freq = 0 #midium rare
max_freq = 0

MAX_TEMP=66000
ONCE_TIME=30

def init_fan_gpio():
    os.popen("gpio mode 41 out")

def fan_on():
    init_fan_gpio()
    os.popen("gpio write 41 1")
    time.sleep(ONCE_TIME)

def fan_off():
    init_fan_gpio()
    os.popen("gpio write 41 0")


def isDigit(x):
    try:
        float(x)
        return True
    except ValueError:
        return False


def cpu_infos():
    global cpus
    global mid_freq
    global max_freq

    cpus = glob.glob('/sys/devices/system/cpu/cpu[0-9]')
    cpus.sort()
#/sys/devices/system/cpu/cpu0/cpufreq/scaling_available_frequencies
#/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor

    scaling_available_freq = open(os.path.join(cpus[0],"cpufreq/scaling_available_frequencies"),"r").read()
    scaling_available_freq = scaling_available_freq.strip("\n")
    scaling_available_freqs = scaling_available_freq.split(" ")

    for var in scaling_available_freqs:
        if isDigit(var):
            if(int(var) > 1080000):
                if(mid_freq == 0):
                    mid_freq = int(var)
            max_freq = int(var)


def set_cpu_on_off(cpu_num):
    global cpus
    if isDigit(cpu_num) == False:
        cpu_num = 999
    else:
        cpu_num = int(cpu_num)

    if cpu_num <= 0: 
        cpu_num = 999
    
    on_off = 1
    start = 1
    for var in cpus:
        online_f = os.path.join(var,"online")
        if start > cpu_num:
            on_off = 0
        
        subprocess.run( "echo %s | sudo tee  %s" %(on_off,online_f),shell=True)
        start = start + 1


def set_gov(gov):
    global cpus
    for var in cpus:
        gov_f = os.path.join(var,"cpufreq/scaling_governor")
        #print(gov_f)
        subprocess.run( "echo %s | sudo tee  %s" %(gov,gov_f),shell=True)
    
def set_performance(scale):
    global cpus
    global mid_freq
    global max_freq

    freq = mid_freq
    if scale =="mid":
        freq = mid_freq
    elif scale =="max":
        freq = max_freq
  
    for var in cpus:
        _f = os.path.join(var,"cpufreq/scaling_max_freq")
        #print(_f)
        subprocess.run( "echo %s | sudo tee  %s" %(freq,_f),shell=True)
  
def set_gpu(gov,hz):
	gpu_sys_path = "/sys/devices/platform/ff9a0000.gpu/devfreq/ff9a0000.gpu"
	gpu_gov_path = os.path.join(gpu_sys_path,"governor")
	gpu_freq_path = os.path.join(gpu_sys_path,"max_freq")
	try:
	    subprocess.run("echo %s | sudo tee %s" %(gov,gpu_gov_path),shell=True)
	    subprocess.run("echo %d | sudo tee %s" %(hz, gpu_freq_path),shell=True)
	except:
	    print("set gpu failed")
		
	
def fan_loop():
    while True:
        temps = glob.glob('/sys/class/thermal/thermal_zone[0-9]/')
        temps.sort()
        for var in temps:
            _f = os.path.join(var,"temp")
            #print( open(_f).read().strip("\n") )
            _t = open(_f).read().strip("\n")
            if isDigit(_t):
                if int(_t) > MAX_TEMP:
                    fan_on()
                    fan_off()
        
        time.sleep(5)


def main(argv):
    global cpus
    scale = 'mid'
    gov   = 'ondemand'
    cpu_num = 2
    try:
        opts, args = getopt.getopt(argv,"hs:g:c:",["scale=","governor=","cpu="])
    except getopt.GetoptError:
        print ('me.py -s <scale> -g <governor> -c <working cpu num>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print ('me.py -s <scale> -g <governor> -c <working cpu num>')
            sys.exit()
        elif opt in ("-s", "--scale"):
            scale = arg
        elif opt in ("-g","--governor"):
            gov = arg
        elif opt in ("-c","--cpu"):
            cpu_num = arg

    print ('Scale is ', scale,"Gov is ",gov,"Cpu num is",cpu_num)

    init_fan_gpio()
    cpu_infos()
    set_gov(gov)
    set_performance(scale)
    set_cpu_on_off(cpu_num)    
    #set_gpu("simple_ondemand",200000000)
    fan_loop()


if __name__ == "__main__":
   main(sys.argv[1:])

